# -*- coding: UTF-8 -*-
from __future__ import division
from django import template

register = template.Library()


@register.filter(name='multiply')
def multiply(value, arg):
    """
    Multiplies the arg and the value

    :param value:
    :param arg:
    :return:
    """
    if value:
        value = "%s" % value
        if type(value) is str and len(value) > 0:
            return float(value.replace(",", ".")) * float(arg)

    return None


@register.filter(name='subtract')
def subtract(value, arg):
    """
    Subtracts the arg from the value

    :param value:
    :param arg:
    :return:
    """
    value = nonetozero(value)
    arg = nonetozero(arg)
    return int(value) - int(arg)


@register.filter(name='divide')
def divide(value, arg):
    """
    Divides the value by the arg

    :param value:
    :param arg:
    :return:
    """
    if value:
        return value / arg
    else:
        return None


@register.filter(name='to_int')
def to_int(value):
    """
    Parses a string to int value

    :param value:
    :return:
    """
    return int(value) if value else 0


@register.filter(name="currency")
def currency(value):
    """
    Converts the number to an €-amount
    """
    if value:
        return (("%.2f" % round(value, 2)) + "€").replace(".", ",")
    else:
        return "-"



# -*- coding: UTF-8 -*-
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit
from django.utils.translation import ugettext_lazy as _


class RequestInFormKwargsMixin(object):
    """
    Injects the request in the form.
    Attention: Have to be removed in the init of the form (via .pop())
    """
    def get_form_kwargs(self):
        kwargs = super(RequestInFormKwargsMixin, self).get_form_kwargs()
        kwargs.update({'request': self.request})
        return kwargs


class CrispyLayoutFormMixin(object):
    """
    Styles the form in bootstrap style
    """
    def __init__(self, *args, **kwargs):
        # Crispy
        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal form-bordered form-row-stripped'
        self.helper.form_method = 'post'
        self.helper.add_input(Submit('submit_button', _(u'Speichern')))
        self.helper.form_tag = True
        self.helper.label_class = 'col-md-3'
        self.helper.field_class = 'col-md-9'
        self.helper.label_size = ' col-md-offset-3'

        super(CrispyLayoutFormMixin, self).__init__(*args, **kwargs)



# import os
#
# import django
# from django.core.exceptions import ImproperlyConfigured
# from django.test import TestCase, override_settings
#
# from ai.services.custom_scrubber import AbstractScrubbingService
#
#
# class AbstractScrubbingServiceTest(TestCase):
#
#     def setUp(self):
#         # TODO this will not for because of not loaded apps. Needs to be fixed at some point...
#
#         self.service = AbstractScrubbingService()
#
#     @override_settings(DEBUG=False)
#     def test_scrubber_debug_mode_needs_to_be_active(self):
#         self.assertEqual(self.service.process(), False)
#
#     @override_settings(DEBUG=True, INSTALLED_APPS=[])
#     def test_scrubber_needs_to_be_installed(self):
#         self.assertEqual(self.service.process(), False)



# -*- coding: utf-8 -*-
from django.core.files.storage import FileSystemStorage
from django.conf import settings

from ai.utils.antivirus import AVException
from .scanner import AVScanner

av_scanner = AVScanner()

USE_SCANNER = settings.ANTIVIRUS_ENABLED if hasattr(settings, 'ANTIVIRUS_ENABLED') else True

class AVFileSystemStorage(FileSystemStorage):
    parallel = True
    def save(self, name, content, max_length=None):
        if USE_SCANNER:
            av_result = av_scanner.has_virus(content, parallel=self.parallel)
            if av_result[0] is False:
                return super(AVFileSystemStorage, self).save(name, content, max_length)
            raise AVException(av_result[1])
        else:
            return super(AVFileSystemStorage, self).save(name, content, max_length)

class SingleCoreAVFileSystemStorage(AVFileSystemStorage):
    """
    Same as AVFileSystemStorage, but doesn't run multiple scanners in parallel
    """
    parallel = False



from django.core import mail
from django.core.exceptions import ImproperlyConfigured
from django.core.mail import EmailMultiAlternatives
from unittest import TestCase

from ai.mail.services.tests import EmailTestService, EmailTestServiceQuerySet


class EmailServiceTest(TestCase):

    @classmethod
    def setUpClass(cls):
        super(EmailServiceTest, cls).setUpClass()

        # Initialize django settings
        from django.conf import settings
        try:
            settings_configured = settings.configured
        except ImproperlyConfigured:
            settings_configured = False
        if not settings_configured:
            settings.configure()

        # Mail data
        cls.subject = 'Super great email (definitely not spam!)'
        cls.to = 'spam@world.com'
        cls.cc = 'spam-copy@world.com'
        cls.bcc = 'spam-secret@world.com'
        cls.content_part = 'body part'
        cls.text_content = 'I\'m the %s.' % cls.content_part
        cls.html_content = '<html>I\'m the %s.</html>' % cls.content_part

        cls.other_mail_subject = 'Other mail subject'

        # Initialize django outbox
        mail.outbox = []

        # Instances
        cls.ets = EmailTestService()

    def setUp(self):
        super().setUp()

        # Create email per test case
        email = EmailMultiAlternatives(self.subject, self.text_content, to=[self.to], cc=[self.cc], bcc=[self.bcc])
        email.attach_alternative(self.html_content, "text/html")
        mail.outbox.append(email)

        # Create second email per test case
        email = EmailMultiAlternatives(self.other_mail_subject, self.text_content, to=['to@world.com'],
                                       cc=['cc@world.com'], bcc=['bcc@world.com'])
        email.attach_alternative(self.html_content, "text/html")
        mail.outbox.append(email)

    def tearDown(self) -> None:
        super(EmailServiceTest, self).tearDown()
        # Emtpy mailbox for next test
        mail.outbox = []

    def test_outbox_is_updated(self):
        self.assertEqual(self.ets.all().count(), 2)
        # Create third mail
        email = EmailMultiAlternatives(self.other_mail_subject, self.text_content, to=['to@world.com'],
                                       cc=['cc@world.com'], bcc=['bcc@world.com'])
        email.attach_alternative(self.html_content, "text/html")
        mail.outbox.append(email)
        self.assertEqual(self.ets.all().count(), 3)

    def test_ensure_outbox_is_loaded(self):
        ets = EmailTestService()
        self.assertEqual(ets._outbox, None)
        ets._ensure_outbox_is_loaded()
        self.assertIsInstance(ets._outbox, list)

    def test_reload(self):
        ets = EmailTestService()
        self.assertEqual(ets._outbox, None)
        ets.reload()
        self.assertIsInstance(ets._outbox, list)

    def test_filter_no_params(self):
        self.assertRaises(ValueError, self.ets.filter)

    def test_filter_to_valid(self):
        qs = self.ets.filter(to=self.to)
        self.assertEqual(qs.count(), 1)

    def test_filter_cc_valid(self):
        qs = self.ets.filter(cc=self.cc)
        self.assertEqual(qs.count(), 1)

    def test_filter_bcc_valid(self):
        qs = self.ets.filter(bcc=self.bcc)
        self.assertEqual(qs.count(), 1)

    def test_filter_subject_valid(self):
        qs = self.ets.filter(subject=self.subject)
        self.assertEqual(qs.count(), 1)

    def test_filter_to_invalid(self):
        qs = self.ets.filter(to='not-my@mail.com')
        self.assertEqual(qs.count(), 0)

    def test_filter_cc_invalid(self):
        qs = self.ets.filter(cc='not-my@mail.com')
        self.assertEqual(qs.count(), 0)

    def test_filter_bcc_invalid(self):
        qs = self.ets.filter(bcc='not-my@mail.com')
        self.assertEqual(qs.count(), 0)

    def test_filter_subject_invalid(self):
        qs = self.ets.filter(subject='Not my subject')
        self.assertEqual(qs.count(), 0)

    def test_filter_to_cc_valid(self):
        qs = self.ets.filter(to=self.to, cc=self.cc)
        self.assertEqual(qs.count(), 1)

    def test_filter_to_cc_bcc_valid(self):
        qs = self.ets.filter(to=self.to, cc=self.cc, bcc=self.bcc)
        self.assertEqual(qs.count(), 1)

    def test_filter_to_cc_bcc_subject_valid(self):
        qs = self.ets.filter(to=self.to, cc=self.cc, bcc=self.bcc, subject=self.subject)
        self.assertEqual(qs.count(), 1)

    def test_filter_to_cc_bcc_subject_invalid(self):
        qs = self.ets.filter(to=self.to, cc=self.cc, bcc=self.bcc, subject='Not my subject')
        self.assertEqual(qs.count(), 0)

    def test_all(self):
        # Assertion
        self.assertEqual(self.ets.all().count(), 2)

    def test_validate_lookup_cache_contains_one_element_true(self):
        # Assertion (we have two mails so it's not equal to 1)
        self.assertEqual(self.ets.all().count(), 2)
        self.assertRaises(RuntimeError, self.ets.all()._validate_lookup_cache_contains_one_element)

    def test_validate_lookup_cache_contains_one_element_false(self):
        self.assertEqual(self.ets.filter(subject=self.subject).count(), 1)
        self.assertEqual(self.ets.filter(subject=self.subject)._validate_lookup_cache_contains_one_element(),
                         None)

    def test_ensure_matching_list_was_populated_false(self):
        self.assertRaises(RuntimeError, EmailTestServiceQuerySet()._ensure_matching_list_was_populated)

    def test_ensure_matching_list_was_populated_true(self):
        self.assertEqual(self.ets.all()._ensure_matching_list_was_populated(), None)

    def test_get_html_content(self):
        self.assertEqual(self.ets.filter(subject=self.subject)._get_html_content(), self.html_content)

    def test_get_txt_content(self):
        self.assertEqual(self.ets.filter(subject=self.subject)._get_txt_content(), self.text_content)

    def test_one_true(self):
        self.assertEqual(self.ets.filter(subject=self.subject).one(), True)

    def test_one_false(self):
        self.assertEqual(self.ets.all().one(), False)

    def test_count(self):
        self.assertEqual(self.ets.all().count(), 2)
        self.assertEqual(self.ets.filter(subject=self.subject).count(), 1)
        self.assertEqual(self.ets.filter(subject='Not my subject').count(), 0)

    def test_first(self):
        self.assertEqual(self.ets.all().first().subject, self.subject)

    def test_last(self):
        self.assertEqual(self.ets.all().last().subject, self.other_mail_subject)

    def test_assert_one_true(self):
        self.ets.filter(subject=self.subject).assert_one()

    def test_assert_one_false(self):
        self.assertRaises(AssertionError, self.ets.all().assert_one)

    def test_assert_quantity_true(self):
        self.ets.filter(subject=self.subject).assert_quantity(1)

    def test_assert_quantity_false(self):
        self.assertRaises(AssertionError, self.ets.filter(subject=self.subject).assert_quantity, 0)

    def test_assert_subject_true(self):
        self.ets.filter(subject=self.subject).assert_subject(self.subject)

    def test_assert_subject_false(self):
        self.assertRaises(AssertionError, self.ets.filter(subject=self.subject).assert_subject, self.other_mail_subject)

    def test_assert_body_contains_true(self):
        self.ets.filter(subject=self.subject).assert_body_contains(self.content_part)

    def test_assert_body_contains_false(self):
        self.assertRaises(AssertionError, self.ets.filter(subject=self.subject).assert_body_contains, 'Not in here!')

    def test_assert_body_contains_not_true(self):
        self.ets.filter(subject=self.subject).assert_body_contains_not('Not in here!')

    def test_assert_body_contains_not_false(self):
        self.assertRaises(AssertionError, self.ets.filter(subject=self.subject).assert_body_contains_not,
                          self.content_part)



def round_up_to_decimal(value, precision: float = 0.5):
    """
    Helper function to round a given value to a specific precision, for example *.5
    So 5.4 will be rounded to 5.5, 0.9 to 1
    :param value:
    :param precision:
    :return:
    """
    return round(precision * round(float(value) / precision), 1)




"""
Stores the given user as creator or editor of the given object
"""
def log_whodid(obj, user):
    if hasattr(obj, 'created_by') and obj.created_by is None:
        obj.created_by = user

    if hasattr(obj, 'lastmodified_by'):
        obj.lastmodified_by = user



from django.contrib import admin


class ReadOnlyAdmin(admin.ModelAdmin):
    """
    Class for being extended by ModelAdmin-classes.
    Disables all create, delete or edit functionality in the regular admin.
    """

    def get_readonly_fields(self, request, obj=None):
        if obj:
            self.readonly_fields = [field.name for field in self.opts.local_fields] + \
                                   [field.name for field in self.opts.local_many_to_many]
        return self.readonly_fields

    def changeform_view(self, request, object_id=None, form_url='', extra_context=None):
        extra_context = extra_context or {}
        extra_context['show_save_and_continue'] = False
        extra_context['show_save'] = False
        return super(ReadOnlyAdmin, self).changeform_view(request, object_id, extra_context=extra_context)

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class EditableOnlyAdmin(admin.ModelAdmin):
    """
    Class for being extended by ModelAdmin-classes.
    Disables all create and delete functionality so all records can only be edited.
    """

    def has_delete_permission(self, request, obj=None):
        return False

    def get_actions(self, request):
        # Disable delete
        actions = super(EditableOnlyAdmin, self).get_actions(request)
        del actions['delete_selected']
        return actions

    def has_add_permission(self, request):
        return False


class ReadOnlyTabularInline(admin.TabularInline):
    """
    Class for being extended by TabularInline-classes.
    Disables all create, delete or edit functionality in the tabular inline admin.
    """
    can_delete = False

    def has_add_permission(self, request):
        return False

    def get_readonly_fields(self, request, obj=None):
        result = list(set(
            [field.name for field in self.opts.local_fields] +
            [field.name for field in self.opts.local_many_to_many]
        ))
        result.remove('id')
        return result



# -*- coding: utf-8 -*-
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import ugettext_lazy as _

from .storage import USE_SCANNER
from .scanner import AVScanner

scanner = AVScanner()


def file_validator(f):
    if USE_SCANNER:
        has_virus, name = scanner.has_virus(f.file)
        if has_virus:
            raise ValidationError(_('In dieser Datei wurde ein Virus erkannt.'))
            # raise ValidationError(_('Virus "{}" wurde erkannt.').format(name))


class AVProtectedFileField(models.FileField):
    def __init__(self, *args, **kwargs):
        super(AVProtectedFileField, self).__init__(*args, **kwargs)
        self.validators.append(file_validator)


class AVProtectedImageField(models.ImageField):
    def __init__(self, *args, **kwargs):
        super(AVProtectedImageField, self).__init__(*args, **kwargs)
        self.validators.append(file_validator)



# -*- coding: UTF-8 -*-
from builtins import str


def get_name_by_value(choices_tuple, input_val):
    for val, name in choices_tuple:
        if val == input_val:
            try:
                return str(name)
            except UnicodeDecodeError:
                return name
    return False



# -*- coding: UTF-8 -*-
from __future__ import division
from datetime import date
import datetime

import pytz
from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.utils.timezone import utc
import time


def add_months(sourcedate, months):
    return sourcedate + relativedelta(months=months)


def add_days(sourcedate, days):
    return sourcedate + relativedelta(days=days)


def add_minutes(sourcedate, minutes):
    return sourcedate + relativedelta(minutes=minutes)


def get_next_month():
    dt = date.today()
    return add_months(dt, 1)


def diff_month(d1, d2):
    return (d1.year - d2.year) * 12 + d1.month - d2.month


def first_day_of_month(d):
    return date(d.year, d.month, 1)


def get_current_datetime():
    return datetime.datetime.utcnow().replace(tzinfo=utc)


def get_formatted_date_str(dt):
    return dt.strftime('%d.%m.%Y')


def get_seconds(string_time):
    duration = time.strptime(string_time, "%H:%M:%S")

    hour = duration.tm_hour
    min = duration.tm_min
    sec = duration.tm_sec
    ##### CONVERT TO SECONDS
    total_seconds = ((hour * 60) + min) * 60 + sec

    return total_seconds


def get_time_from_seconds(seconds):
    ### CONVERT TO TIME
    new_hor = seconds // 3600
    new_minu = (seconds - (new_hor * 3600)) // 60
    new_seg = seconds - ((new_hor * 3600) + (new_minu * 60))
    result = "%02d:%02d:%02d" % (new_hor, new_minu, new_seg)
    return result


def datetime_format(datetime, dt_format):  # Uses strftime, but considers timezone (only for datetime objects)
    try:
        dt_format = datetime.astimezone(tz=pytz.timezone(settings.TIME_ZONE)).strftime(dt_format)
    except:
        dt_format = datetime.strftime(dt_format)
    return dt_format



# -*- coding: utf-8 -*-

from .exception import AVException
from .storage import AVFileSystemStorage, SingleCoreAVFileSystemStorage
from .scanner import AVScanner
from .field import AVProtectedFileField, AVProtectedImageField



# # Generate a human readable 'random' password
## password  will be generated in the form 'word'+digits+'word'
## eg.,nice137pass
## parameters: number of 'characters' , number of 'digits'
## Pradeep Kishore Gowda <pradeep at btbytes.com >
## License : GPL
## Date : 2005.April.15
## Revision 1.2
## ChangeLog:
## 1.1 - fixed typos
## 1.2 - renamed functions _apart & _npart to a_part & n_part as zope does not allow functions to
## start with _
from __future__ import division
def generate_password(alpha=8, numeric=4):
    """
    returns a human-readble password (say rol86din instead of
    a difficult to remember K8Yn9muL )
    """
    import string
    import random

    vowels = ['a', 'e', 'i', 'o', 'u']
    consonants = [a for a in string.ascii_lowercase if a not in vowels]
    digits = string.digits

    ####utility functions
    def a_part(slen):
        ret = ''
        for i in range(slen):
            if i % 2 == 0:
                randid = random.randint(0, 20)  # number of consonants
                ret += consonants[randid]
            else:
                randid = random.randint(0, 4)  # number of vowels
                ret += vowels[randid]
        return ret

    def n_part(slen):
        ret = ''
        for i in range(slen):
            randid = random.randint(0, 9)  # number of digits
            ret += digits[randid]
        return ret

    fpl = alpha // 2
    if alpha % 2:
        fpl = (alpha // 2) + 1
    lpl = alpha - fpl

    start = a_part(fpl)
    mid = n_part(numeric)
    end = a_part(lpl)

    myPass = "%s%s%s" % (start, mid, end)

    return myPass.capitalize()



import threading
import time
from unittest import TestCase

from ai.middleware.current_user import CurrentUserMiddleware

from mock import Mock


class CurrentUserMiddlewareTest(TestCase):
    def test_current_user_is_None_if_no_user_given(self):
        self.assertIsNone(CurrentUserMiddleware.get_current_user())

    def test_current_user_is_None_if_request_user_is_None(self):
        set_current_user(user=None)
        self.assertIsNone(CurrentUserMiddleware.get_current_user())

    def test_current_user_is_same_as_request_user(self):
        new_user = Mock(user_name='test_user')
        set_current_user(user=new_user)
        current_user = CurrentUserMiddleware.get_current_user()

        self.assertIsNotNone(current_user)
        self.assertEqual(current_user, new_user)
        self.assertEqual(current_user.user_name, 'test_user')

    def test_current_user_is_thread_safe(self):
        user1 = Mock(user_name='user1')
        user2 = Mock(user_name='user2')
        current_users = []

        first_thread = threading.Thread(target=set_current_user, args=(user1, 0, 5, current_users))
        second_thread = threading.Thread(target=set_current_user, args=(user2, 3, 0, current_users))
        first_thread.start()
        second_thread.start()
        first_thread.join()
        second_thread.join()

        self.assertEqual(current_users[0], user2)
        self.assertEqual(current_users[0].user_name, 'user2')
        self.assertEqual(current_users[1], user1)
        self.assertEqual(current_users[1].user_name, 'user1')


def set_current_user(user=None, delay_before_request=0, delay_after_request=0, current_users=None):
    request = Mock()
    request.user = user
    middleware = CurrentUserMiddleware()

    if delay_before_request:
        time.sleep(delay_before_request)
    middleware.process_request(request)
    if delay_after_request > 0:
        time.sleep(delay_after_request)

    if current_users is not None:
        current_users.append(CurrentUserMiddleware.get_current_user())



# -*- coding: utf-8 -*-
import os
import tempfile
from multiav.core import CMultiAV, AV_SPEED_MEDIUM


class AVScanner(object):
    def __init__(self):
        self.multi_av = CMultiAV("%s/cmultiav.cfg" % os.path.dirname(__file__))

    def has_virus(self, file, parallel=True):
        with tempfile.NamedTemporaryFile(delete=False) as temporaryfile:
            temporaryfile.write(file.read())
            temporaryfile.close()
            permission = 0o664 # PEP 3127: octal literals
            os.chmod(temporaryfile.name, permission)  # clamav needs permission to scan
            if parallel:
                # TODO: Handle the case for when no scanner is installed ...
                # Currently we say that nothing was found...
                # The single core case works as expected
                ret = self.multi_av.scan(temporaryfile.name, AV_SPEED_MEDIUM)
            else:

                try:
                    ret = self.multi_av.single_scan(temporaryfile.name, AV_SPEED_MEDIUM)
                except OSError as err:
                    # It would seem a scanner is not installed...
                    return True, "No virus scanner was found on the system..."

            os.unlink(temporaryfile.name)
            for x in ret.values():
                if x != {}:
                    # all is lost as soon as one scanner finds something
                    return True, ret
            return False, None



# -*- coding: utf-8 -*-
from django import template
from django.utils import timezone

register = template.Library()


@register.simple_tag
def js_versiontag():
    return "?s=%s" % timezone.now().microsecond



import datetime
from unittest import TestCase

from django.core.exceptions import ImproperlyConfigured
from django.test.utils import override_settings
from freezegun import freeze_time

from ai.utils.date import date_month_delta, get_start_and_end_date_from_calendar_week, tz_today


class DateUtilTest(TestCase):
    TEST_CURRENT_YEAR = 2017

    @classmethod
    def setUpClass(cls):
        super(DateUtilTest, cls).setUpClass()

        # Initialize django settings
        from django.conf import settings
        try:
            settings_configured = settings.configured
        except ImproperlyConfigured:
            settings_configured = False
        if not settings_configured:
            settings.configure()

    def setUp(self):
        # BaseTest setup
        super(DateUtilTest, self).setUp()

    def test_get_start_and_end_date_from_calendar_week(self):
        monday, sunday = get_start_and_end_date_from_calendar_week(2016, 52)
        self.assertEquals(monday, datetime.date(year=2016, month=12, day=26))

        monday, sunday = get_start_and_end_date_from_calendar_week(2018, 1)
        self.assertEquals(monday, datetime.date(year=2018, month=1, day=1))

        monday, sunday = get_start_and_end_date_from_calendar_week(2017, 30)
        self.assertEquals(monday, datetime.date(year=2017, month=7, day=24))

    def test_date_month_delta(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=2, day=1)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=3, day=1)
        self.assertEqual(date_month_delta(start_date, end_date), 1)

    def test_date_month_delta_half_month(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=15)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=5, day=1)
        self.assertAlmostEqual(date_month_delta(start_date, end_date), 0.5, 1)

    def test_date_month_delta_one_and_a_half_month(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=15)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=6, day=1)
        self.assertAlmostEqual(date_month_delta(start_date, end_date), 1.5, 1)

    def test_date_month_delta_full_year(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=1, day=1)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR + 1, month=1, day=1)
        self.assertEqual(date_month_delta(start_date, end_date), 12)

    def test_date_month_delta_two_years(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=1, day=1)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR + 2, month=7, day=1)
        self.assertEqual(date_month_delta(start_date, end_date), 30)

    def test_date_month_delta_no_difference(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=15)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=15)
        self.assertEqual(date_month_delta(start_date, end_date), 0)

    def test_date_month_delta_one_day(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=15)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=16)
        self.assertEqual(date_month_delta(start_date, end_date), 1 / 30)

    def test_date_month_delta_start_greater_then_end_date(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=15)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=14)
        self.assertRaises(NotImplementedError, date_month_delta, start_date, end_date)

    @override_settings(USE_TZ=True)
    def test_tz_today_as_object_tz_active(self):
        frozen_date = datetime.datetime(year=2019, month=9, day=19, hour=10)
        with freeze_time(frozen_date):
            self.assertEqual(tz_today(), frozen_date.date())

    def test_tz_today_as_object_tz_not_active(self):
        frozen_date = datetime.datetime(year=2019, month=9, day=19, hour=10)
        with freeze_time(frozen_date):
            self.assertEqual(tz_today(), frozen_date.date())

    @override_settings(USE_TZ=True)
    def test_tz_today_as_str(self):
        frozen_date = datetime.datetime(year=2019, month=9, day=19, hour=10)
        with freeze_time(frozen_date):
            self.assertEqual(tz_today('%d.%m.%Y'), '19.09.2019')



# -*- coding: UTF-8 -*-
import re
import sys
from operator import setitem
from builtins import str

from django.contrib.humanize.templatetags.humanize import intcomma
from django.template.defaultfilters import floatformat

from ai.converter.dateconverter import datetime_format


def number_to_text(value):
    if value == 0:
        return u"first"
    elif value == 1:
        return u"second"
    else:
        return u"third"


def distinct(l):
    d = {}
    map(setitem, (d,)*len(l), l, [])
    return d.keys()


def slugify_file_name(file_name, length=40):
    """
        Translit and slugify file name.
    """
    result = ""
    try:
        from django.template.defaultfilters import slugify
        from django.utils.encoding import smart_str
        nameext = file_name.rsplit('.',1)
        name = ''
        ext = ''
        if nameext:
            name = nameext[0]
            if len(nameext) > 1:
                ext = nameext[1]
        #name = name.encode('latin1')
        name = smart_str(slugify(name).replace('-', '_'))
        ext = smart_str(slugify(ext))
        result = '%s%s%s' % (name[:length], "." if ext else "", ext)
        #print "after: %s" % result
    except:
        print("Unexpected error:", sys.exc_info()[0])
    return result


def replace_link_pattern(text):
    text = re.sub(r"(^|[\n ])([\w]+?://[\w]+[^ \"\n\r\t< ]*)", "\g<1><a href=\"\g<2>\" target=\"_blank\">\g<2></a>", text);
    text = re.sub(r"(^|[\n ])((www|ftp)\.[^ \"\t\n\r< ]*)", "\g<1><a href=\"http://\g<2>\" target=\"_blank\">\g<2></a>", text)
    return text


def smart_truncate(text, max_length=100, suffix='...'):
    """
    Returns a string of at most `max_length` characters, cutting
    only at word-boundaries. If the string was truncated, `suffix`
    will be appended.
    """

    if text is None:
        return ''

    # Return the string itself if length is smaller or equal to the limit
    if len(text) <= max_length:
        return text

    # Cut the string
    value = text[:max_length]

    # Break into words and remove the last
    words = value.split(' ')[:-1]

    # Join the words and return
    return ' '.join(words) + suffix


def restore_windows1252controls(s):
    """
    Replace C1 control characters in the Unicode string s by the
    characters at the corresponding code points in Windows-1252, where
    possible.
    """
    def to_windows1252(match):
        try:
            return chr(ord(match.group(0))).decode('windows-1252')
        except UnicodeDecodeError:
            # No character at the corresponding code point: remove it.
            return ''
    pattern = r'[\u0080-\u0099]'
    try:
        pattern = pattern.decode('raw_unicode_escape')
    except AttributeError as e:
        # Python 3
        # PEP 414: ur'...' prefix replaced wit just r'...'
        pass
    return re.sub(pattern, to_windows1252, s)


def float_to_string(value, replacement="0,00"):
    def localize_float(value):
        return value.replace('.', ',')
    return localize_float("%.2f" % value) if value is not None else replacement


def date_to_string(value, replacement="-", format="%d.%m.%Y"):
    return value.strftime(format) if value is not None else replacement


def datetime_to_string(value, replacement="-", format="%d.%m.%Y %H:%M"):
    return datetime_format(value, format) if value is not None else replacement


def number_to_string(value, decimal_digits=0, replacement="-"):
    return intcomma(floatformat(value, decimal_digits)) if value is not None else replacement


def string_or_none_to_string(value, replacement="-"):
    return value if value is not None else replacement


def encode_to_xml(text):
    text_str = str(text)
    text_str = text_str.replace('&', '&amp;')
    text_str = text_str.replace('<', '&lt;')
    text_str = text_str.replace('>', '&gt;')

    return text_str



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import ast
import re

from setuptools import setup, find_packages

# Dependencies
requirements = [
    'django-crispy-forms>=1.4.0',
    'Django>=1.7.0',
    'bleach>=1.4.2',
    'python-dateutil>=2.5.3',
    'future>=0.16',
    'freezegun',
    'pytest-cov',
    'pytest-django',
    'mock'
]

# Parse version
_version_re = re.compile(r"__version__\s+=\s+(.*)")
with open("ai/__init__.py", "rb") as f:
    version = str(
        ast.literal_eval(_version_re.search(f.read().decode("utf-8")).group(1))
    )

setup(
    name='ai-django-core',
    version=version,
    author=u'Ambient Innovation: GmbH',
    author_email='hello@ambient-innovation.com',
    packages=find_packages(),
    include_package_data=True,
    url='https://gitlab.ambient-innovation.com/ai/ai-django-core.git',
    license="The MIT License (MIT)",
    description='Lots of helper functions and useful widgets.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    zip_safe=False,
    dependency_links=['https://github.com/ambient-innovation/multiav/master/#egg=multiav', ],
    install_requires=requirements
)



import zlib
import hashlib


def get_filename_without_ending(filepath):
    """
    Returns the filename without extension
    :param filename:
    :return:
    """

    # if filename has filepath parts
    if '/' in filepath:
        filename = filepath.rsplit('/')[-1]
    else:
        filename = filepath

    return filename.rsplit('.', 1)[0]


def crc(fileName):
    prev = 0
    for eachLine in open(fileName,"rb"):
        prev = zlib.crc32(eachLine, prev)
    return "%X"%(prev & 0xFFFFFFFF)


def md5_checksum(filePath):
    """
    Returns the md5 checksum of the file from the given filepath
    :param filePath:
    :return:
    """
    fh = open(filePath, 'rb')
    m = hashlib.md5()
    while True:
        data = fh.read(8192)
        if not data:
            break
        m.update(data)
    return m.hexdigest()



# -*- coding: utf-8 -*-
from django.shortcuts import redirect, render


class _FormsetMixin(object):
    """
    Do NOT use directly. Do use FormsetUpdateViewMixin or FormsetCreateViewMixin
    """
    def get_formset_kwargs(self):
        # may be overridden or extended
        return dict(instance=self.object)

    def get_context_data(self, **kwargs):
        context = super(_FormsetMixin, self).get_context_data(**kwargs)
        context['formset'] = self.formset_class(**self.get_formset_kwargs())
        return context

    def post(self, request, *args, **kwargs):
        form_class = self.get_form_class()
        form = form_class(**self.get_form_kwargs())
        formset = self.formset_class(request.POST, request.FILES, **self.get_formset_kwargs())

        if form.is_valid() and formset.is_valid():
            self.object = form.save()
            formset.instance = self.object
            formset.save()

            if hasattr(self, 'additional_is_valid'):
                self.additional_is_valid(form, formset)

            return redirect(self.get_success_url())
        # Get all context data
        context = self.get_context_data()
        # Update form and formset variables
        context['form'] = form
        context['formset'] = formset
        # Pass all data to template
        return render(request, self.get_template_names(), context)


class FormsetUpdateViewMixin(_FormsetMixin):
    """
    Can be used to validate and save a formset.
    Usage is similar to regular UpdateView:

    class MandantSettingsUpdate(FormsetUpdateViewMixin, generic.UpdateView):
        form_class = MandantSettingsForm
        template_name = 'mandant/mandantsettings_form.html'
        formset_class = inlineformset_factory(Mandant, MandantProdukttyp, form=MandantProdukttypForm, extra=0, can_delete=False)
        model = Mandant
        success_url = reverse_lazy('mandant:edit-settings')

        def additional_is_valid(self, form, formset):
            messages.add_message(self.request, messages.INFO, 'Einstellungen wurden erfolgreich aktualisiert.')

    """

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(FormsetUpdateViewMixin, self).post(request, *args, **kwargs)


class FormsetCreateViewMixin(_FormsetMixin):
    """
    Can be used to validate and save a formset.
    Usage is similar to regular CreateView. See FormsetUpdateViewMixin.
    """

    def post(self, request, *args, **kwargs):
        self.object = None
        return super(FormsetCreateViewMixin, self).post(request, *args, **kwargs)



# -*- coding: utf-8 -*-
import re

from django.conf import settings
from django.core.mail.backends.smtp import EmailBackend as SMTPEmailBackend

EMAIL_BACKEND_DOMAIN_WHITELIST = getattr(settings, 'EMAIL_BACKEND_DOMAIN_WHITELIST', [])
EMAIL_BACKEND_REDIRECT_ADDRESS = getattr(settings, 'EMAIL_BACKEND_REDIRECT_ADDRESS')
REGEX = r'^[\w\-\.]+@(%s)$' % '|'.join(x for x in EMAIL_BACKEND_DOMAIN_WHITELIST).replace('.', '\.')


class WhitelistEmailBackend(SMTPEmailBackend):
    """
    Man kann über die folgenden Settings konfigurieren, ob Mails an _alle_ Adressen verschickt werden.
    Wenn nicht, dann kann eine Umleitung an ein Postfach mit CATCHALL konfiguriert werden.

    EMAIL_BACKEND = 'ai.mail.backends.whitelist_smtp.WhitelistEmailBackend'
    EMAIL_BACKEND_DOMAIN_WHITELIST = ['ambient-innovation.com']
    EMAIL_BACKEND_REDIRECT_ADDRESS = '%s@testuser.ambient-innovation.com'

    Wenn EMAIL_BACKEND_REDIRECT_ADDRESS gesetzt ist, wird eine Mail an xy.z@example.com umgeleitet an:
    xy.z_example.com@testuser.ambient-innovation.com
    """

    @staticmethod
    def whitify_mail_adresses(mail_adress_list):
        allowed_recipients = []
        for to in mail_adress_list:
            if re.search(REGEX, to):
                allowed_recipients.append(to)
            elif EMAIL_BACKEND_REDIRECT_ADDRESS:
                # sende nicht erlaubte Mails an die angegebene Redirect-Adresse (mit CATCHALL)
                allowed_recipients.append(EMAIL_BACKEND_REDIRECT_ADDRESS % to.replace('@', '_'))
        return allowed_recipients

    def send_messages(self, email_messages):
        """
        Checks if email-recipients are in allowed domains and cancels if not.
        Uses regular smtp-sending afterwards.
        """
        for email in email_messages:
            allowed_recipients = self.whitify_mail_adresses(email.to)
            email.to = allowed_recipients
        super(WhitelistEmailBackend, self).send_messages(email_messages)
